@@ -1,6 +1,6 @@
 /*
     scardcontrol.c: sample code to use/test SCardControl() API
-    Copyright (C) 2004-2005   Ludovic Rousseau
+    Copyright (C) 2004-2007   Ludovic Rousseau
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
 */
 
 /*
- * $Id: scardcontrol.c,v 1.7 2007-01-25 20:18:58 rousseau Exp $
+ * $Id: scardcontrol.c 2653 2007-10-15 07:45:32Z rousseau $
  */
 
 #include <stdio.h>
@@ -27,11 +27,16 @@
 #include <unistd.h>
 #include <string.h>
 #include <arpa/inet.h>
+#ifdef __APPLE__
+#include <PCSC/winscard.h>
+#include <PCSC/wintypes.h>
+#else
 #include <winscard.h>
+#endif
 #include <reader.h>
 
-#define VERIFY_PIN
-#undef MODIFY_PIN
+#undef VERIFY_PIN
+#define MODIFY_PIN
 
 #ifndef TRUE
 #define TRUE 1
@@ -61,7 +66,7 @@ int main(int argc, char *argv[])
 	LONG rv;
 	SCARDCONTEXT hContext;
 	DWORD dwReaders;
-	LPSTR mszReaders;
+	LPSTR mszReaders = NULL;
 	char *ptr, **readers = NULL;
 	int nbReaders;
 	SCARDHANDLE hCard;
@@ -69,7 +74,7 @@ int main(int argc, char *argv[])
 	BYTE pbAtr[MAX_ATR_SIZE] = "";
 	char pbReader[MAX_READERNAME] = "";
 	int reader_nb;
-	int i;
+	unsigned int i;
 	unsigned char bSendBuffer[MAX_BUFFER_SIZE];
 	unsigned char bRecvBuffer[MAX_BUFFER_SIZE];
 	DWORD send_length, length;
@@ -89,7 +94,7 @@ int main(int argc, char *argv[])
 #endif
 
 	printf("SCardControl sample code\n");
-	printf("V 1.1 2004-2005, Ludovic Rousseau <ludovic.rousseau@free.fr>\n");
+	printf("V 1.2 2004-2007, Ludovic Rousseau <ludovic.rousseau@free.fr>\n");
 
 	printf("\nTHIS PROGRAM IS NOT DESIGNED AS A TESTING TOOL!\n");
 	printf("Do NOT use it unless you really know what you do.\n\n");
@@ -103,10 +108,7 @@ int main(int argc, char *argv[])
 
 	/* Retrieve the available readers list */
 	rv = SCardListReaders(hContext, NULL, NULL, &dwReaders);
-	if (rv != SCARD_S_SUCCESS)
-	{
-		printf("SCardListReader: %lX\n", rv);
-	}
+	PCSC_ERROR_EXIT(rv, "SCardListReaders")
 
 	mszReaders = malloc(sizeof(char)*dwReaders);
 	if (mszReaders == NULL)
@@ -244,7 +246,7 @@ int main(int argc, char *argv[])
 	/* connect to a reader (even without a card) */
 	dwActiveProtocol = -1;
 	rv = SCardReconnect(hCard, SCARD_SHARE_SHARED,
-		SCARD_PROTOCOL_T0|SCARD_PROTOCOL_T1, SCARD_UNPOWER_CARD,
+		SCARD_PROTOCOL_T0|SCARD_PROTOCOL_T1, SCARD_LEAVE_CARD,
 		&dwActiveProtocol);
 	printf(" Protocol: %ld\n", dwActiveProtocol);
 	PCSC_ERROR_EXIT(rv, "SCardReconnect")
@@ -258,7 +260,7 @@ int main(int argc, char *argv[])
 			pioSendPci = *SCARD_PCI_T1;
 			break;
 		default:
-			printf("Unknown protocol\n");
+			printf("Unknown protocol. No card present?\n");
 			return -1;
 	}
 
@@ -416,6 +418,12 @@ int main(int argc, char *argv[])
 	printf(" Secure modify PIN\n");
 	pin_modify = (PIN_MODIFY_STRUCTURE *)bSendBuffer;
 
+	/* Table for bConfirmPIN and bNumberMessage
+	 * bConfirmPIN = 3, bNumberMessage = 3: "Enter Pin" "New Pin" "Confirm Pin"
+	 * bConfirmPIN = 2, bNumberMessage = 2: "Enter Pin" "New Pin"
+	 * bConfirmPIN = 1, bNumberMessage = 2: "New Pin" "Confirm Pin"
+	 * bConfirmPIN = 0, bNumberMessage = 1: "New Pin"
+	 */
 	/* PC/SC v2.0.2 Part 10 PIN verification data structure */
 	pin_modify -> bTimerOut = 0x00;
 	pin_modify -> bTimerOut2 = 0x00;
@@ -424,11 +432,11 @@ int main(int argc, char *argv[])
 	pin_modify -> bmPINLengthFormat = 0x00;
 	pin_modify -> bInsertionOffsetOld = 0x00; 	/* offset from APDU start */
 	pin_modify -> bInsertionOffsetNew = 0x04;	/* offset from APDU start */
-	pin_modify -> wPINMaxExtraDigit = HOST_TO_CCID_16(0x0404);	/* Min Max */
+	pin_modify -> wPINMaxExtraDigit = HOST_TO_CCID_16(0x0408);	/* Min Max */
 	pin_modify -> bConfirmPIN = 0x03;	/* b0 set = confirmation requested */
 									/* b1 set = current PIN entry requested */
 	pin_modify -> bEntryValidationCondition = 0x02;	/* validation key pressed */
-	pin_modify -> bNumberMessage = 0x03; /* 3 for GemPC Pinpad, 0 for SPR532 */
+	pin_modify -> bNumberMessage = 0x03; /* see table above */
 	pin_modify -> wLangId = HOST_TO_CCID_16(0x0904);
 	pin_modify -> bMsgIndex1 = 0x00;
 	pin_modify -> bMsgIndex2 = 0x00;
@@ -477,7 +485,6 @@ int main(int argc, char *argv[])
 		fd_set fd;
 #endif
 		struct timeval timeout;
-		int i;
 
 		/* old PIN, new PIN, confirmation PIN */
 		/* if the command is aborted we will not read every "PIN" */
@@ -551,8 +558,10 @@ end:
 			rv);
 
 	/* free allocated memory */
-	free(mszReaders);
-	free(readers);
+	if (mszReaders)
+		free(mszReaders);
+	if (readers)
+		free(readers);
 
 	return 0;
 } /* main */
