@@ -18,7 +18,7 @@
 */
 
 /*
- * $Id: ccid.c,v 1.7 2007-01-25 20:18:58 rousseau Exp $
+ * $Id: ccid.c 2596 2007-07-03 16:10:28Z rousseau $
  */
 
 #include <stdio.h>
@@ -73,7 +73,7 @@ int ccid_open_hack(unsigned int reader_index)
 			{
 #define L10N_HEADER_SIZE 5
 #define L10N_STRING_MAX_SIZE 16
-#define L10N_NB_STRING 9
+#define L10N_NB_STRING 10
 
 				unsigned char cmd[L10N_HEADER_SIZE + L10N_NB_STRING * L10N_STRING_MAX_SIZE];
 				unsigned char res[20];
@@ -89,7 +89,8 @@ int ccid_open_hack(unsigned int reader_index)
 					"Delai depasse",
 					"* essai restant",
 					"Inserer carte",
-					"Erreur carte" };
+					"Erreur carte",
+					"PIN bloque" };
 
 				const char *de[] = {
 					"PIN eingeben",
@@ -100,7 +101,8 @@ int ccid_open_hack(unsigned int reader_index)
 					"Zeit abgelaufen",
 					"* Versuche ubrig",
 					"Karte einstecken",
-					"Fehler Karte" };
+					"Fehler Karte",
+					"PIN blockiert" };
 
 				const char *es[] = {
 					"Introducir PIN",
@@ -111,7 +113,8 @@ int ccid_open_hack(unsigned int reader_index)
 					"Tiempo Agotado",
 					"* ensayos quedan",
 					"Introducir Tarj.",
-					"Error en Tarjeta" };
+					"Error en Tarjeta",
+					"PIN bloqueado" };
 
 				const char *it[] = {
 					"Inserire PIN",
@@ -122,7 +125,44 @@ int ccid_open_hack(unsigned int reader_index)
 					"Tempo scaduto",
 					"* prove rimaste",
 					"Inserire Carta",
-					"Errore Carta" };
+					"Errore Carta",
+					"PIN ostruito"};
+
+				const char *pt[] = {
+					"Insira PIN",
+					"Novo PIN",
+					"Conf. novo PIN",
+					"PIN OK",
+					"PIN falhou!",
+					"Tempo expirou",
+					"* tentiv. restam",
+					"Introduza cartao",
+					"Erro cartao",
+					"PIN bloqueado" };
+
+				const char *nl[] = {
+					"Inbrengen code",
+					"Nieuwe code",
+					"Bevestig code",
+					"Code aanvaard",
+					"Foute code",
+					"Time out",
+					"* Nog Pogingen",
+					"Kaart inbrengen",
+					"Kaart fout",
+					"Kaart blok" };
+
+				const char *tr[] = {
+					"PIN Giriniz",
+					"Yeni PIN",
+					"PIN Onayala",
+					"PIN OK",
+					"Yanlis PIN",
+					"Zaman Asimi",
+					"* deneme kaldi",
+					"Karti Takiniz",
+					"Kart Hatasi",
+					"Kart Kilitli" };
 
 				const char *en[] = {
 					"Enter PIN",
@@ -133,7 +173,8 @@ int ccid_open_hack(unsigned int reader_index)
 					"Time Out",
 					"* retries left",
 					"Insert Card",
-					"Card Error" };
+					"Card Error",
+					"PIN blocked" };
 
 				char *lang;
 				const char **l10n;
@@ -151,6 +192,12 @@ int ccid_open_hack(unsigned int reader_index)
 						l10n = es;
 					else if (0 == strncmp(lang, "it", 2))
 						l10n = it;
+					else if (0 == strncmp(lang, "pt", 2))
+						l10n = pt;
+					else if (0 == strncmp(lang, "nl", 2))
+						l10n = nl;
+					else if (0 == strncmp(lang, "tr", 2))
+						l10n = tr;
 					else
 						l10n = en;
 				}
@@ -233,6 +280,38 @@ int ccid_open_hack(unsigned int reader_index)
 			break;
 	}
 
+	/* ICCD type A */
+	if (ICCD_A == ccid_descriptor->bInterfaceProtocol)
+	{
+		unsigned char tmp[MAX_ATR_SIZE];
+		unsigned int n = sizeof(tmp);
+
+		DEBUG_COMM("ICCD type A");
+		CmdPowerOff(reader_index);
+		CmdPowerOn(reader_index, &n, tmp, CCID_CLASS_AUTO_VOLTAGE);
+		CmdPowerOff(reader_index);
+	}
+
+	/* ICCD type B */
+	if (ICCD_B == ccid_descriptor->bInterfaceProtocol)
+	{
+		unsigned char tmp[MAX_ATR_SIZE];
+		unsigned int n = sizeof(tmp);
+
+		DEBUG_COMM("ICCD type B");
+		if (CCID_CLASS_SHORT_APDU ==
+			(ccid_descriptor->dwFeatures & CCID_CLASS_EXCHANGE_MASK))
+		{
+			/* use the extended APDU comm alogorithm */
+			ccid_descriptor->dwFeatures &= ~CCID_CLASS_EXCHANGE_MASK;
+			ccid_descriptor->dwFeatures |= CCID_CLASS_EXTENDED_APDU;
+		}
+
+		CmdPowerOff(reader_index);
+		CmdPowerOn(reader_index, &n, tmp, CCID_CLASS_AUTO_VOLTAGE);
+		CmdPowerOff(reader_index);
+	}
+
 	return 0;
 } /* ccid_open_hack */
 
@@ -244,7 +323,7 @@ int ccid_open_hack(unsigned int reader_index)
 void ccid_error(int error, const char *file, int line, const char *function)
 {
 	const char *text;
-	char var_text[20];
+	char var_text[30];
 
 	switch (error)
 	{
