@@ -18,7 +18,7 @@
 */
 
 /*
- * $Id: ccid.h,v 1.8 2007-01-27 17:06:49 rousseau Exp $
+ * $Id: ccid.h 2685 2007-11-15 23:10:36Z rousseau $
  */
 
 typedef struct
@@ -90,6 +90,11 @@ typedef struct
 	 * Card protocol
 	 */
 	int cardProtocol;
+
+	/*
+	 * bInterfaceProtocol (CCID, ICCD-A, ICCD-B)
+	 */
+	int bInterfaceProtocol;
 } _ccid_descriptor;
 
 /* Features from dwFeatures */
@@ -118,6 +123,10 @@ typedef struct
 #define CCID_COMMAND_FAILED			0x40	/* 01 0000 00 */
 #define CCID_TIME_EXTENSION			0x80	/* 10 0000 00 */
 
+/* bInterfaceProtocol for ICCD */
+#define ICCD_A	1	/* ICCD Version A */
+#define ICCD_B	2	/* ICCD Version B */
+
 /* Product identification for special treatments */
 #define GEMPC433	0x08E64433
 #define GEMPCKEY	0x08E63438
@@ -133,10 +142,12 @@ typedef struct
 #define CHERRYXX33	0x046A0005
 #define CHERRYST2000	0x046A003E
 #define OZ776		0x0B977762
+#define OZ776_7772	0x0B977772
 #define SPR532		0x04E6E003
 #define MYSMARTPAD	0x09BE0002
 #define CHERRYXX44	0x046a0010
 #define CL1356D		0x0B810200
+#define REINER_SCT	0x0C4B0300 
 
 /*
  * The O2Micro OZ776S reader has a wrong USB descriptor
