@@ -18,7 +18,7 @@
 */
 
 /*
- * $Id: ccid_usb.c 2510 2007-05-03 19:21:54Z rousseau $
+ * $Id: ccid_usb.c 2691 2007-11-16 13:26:40Z rousseau $
  */
 
 #define __CCID_USB__
@@ -102,9 +102,6 @@ struct _bogus_firmware
 };
 
 static struct _bogus_firmware Bogus_firmwares[] = {
-#ifndef O2MICRO_OZ776_PATCH
-	{ 0x0b97, 0x7762, 0x0111 },	/* Oz776S */ /* the firmware version is not correct since I do not have received a working reader yet */
-#endif
 	{ 0x04e6, 0xe001, 0x0516 },	/* SCR 331 */
 	{ 0x04e6, 0x5111, 0x0620 },	/* SCR 331-DI */
 	{ 0x04e6, 0x5115, 0x0514 },	/* SCR 335 */
@@ -115,6 +112,13 @@ static struct _bogus_firmware Bogus_firmwares[] = {
 	{ 0x0DC3, 0x1004, 0x0502 },	/* ASE IIIe USBv2 */
 	{ 0x0DC3, 0x1102, 0x0607 },	/* ASE IIIe KB USB */
 	{ 0x058F, 0x9520, 0x0102 },	/* Alcor AU9520-G */
+
+	/* the firmware version is not correct since I do not have received a
+	 * working reader yet */
+	{ 0x0C4B, 0x0300, 0x0101 },	/* Reiner-SCT cyberJack pinpad(a) */
+#ifndef O2MICRO_OZ776_PATCH
+	{ 0x0b97, 0x7762, 0x0111 },	/* Oz776S */
+#endif
 };
 
 /* data rates supported by the secondary slots on the GemCore Pos Pro & SIM Pro */
@@ -633,9 +637,12 @@ static int get_end_points(struct usb_device *dev, _usbDevice *usbdevice)
 {
 	struct usb_interface *usb_interface = NULL;
 	int i;
+#ifdef O2MICRO_OZ776_PATCH
+	int readerID;
+#endif
 
 	/* if multiple interfaces use the first one with CCID class type */
-	for (i=0; i<dev->config->bNumInterfaces; i++)
+	for (i=0; dev->config && i<dev->config->bNumInterfaces; i++)
 	{
 		/* CCID Class? */
 		if (dev->config->interface[i].altsetting->bInterfaceClass == 0xb
@@ -650,9 +657,10 @@ static int get_end_points(struct usb_device *dev, _usbDevice *usbdevice)
 	}
 
 #ifdef O2MICRO_OZ776_PATCH
+	readerID = (dev->descriptor.idVendor << 16) + dev->descriptor.idProduct;
 	if (usb_interface != NULL
-		&& (OZ776 == (dev->descriptor.idVendor << 16)
-		+ dev->descriptor.idProduct)
+		&& ((OZ776 == readerID) || (OZ776_7772 == readerID)
+		|| (REINER_SCT == readerID))
 		&& (0 == usb_interface->altsetting->extralen)) /* this is the bug */
 	{
 		for (i=0; i<usb_interface->altsetting->bNumEndpoints; i++)
