@@ -18,9 +18,11 @@
 */
 
 /*
- * $Id: ccid_usb.h 2292 2007-01-02 16:25:03Z rousseau $
+ * $Id: ccid_usb.h 2427 2007-02-23 16:49:50Z rousseau $
  */
 
+#ifndef __CCID_USB_H__
+#define __CCID_USB_H__
 status_t OpenUSB(unsigned int reader_index, int channel);
 
 status_t OpenUSBByName(unsigned int reader_index, /*@null@*/ char *device);
@@ -37,3 +39,7 @@ status_t CloseUSB(unsigned int reader_index);
 /*@null@*/ struct usb_interface *get_ccid_usb_interface(struct usb_device *dev);
 #endif
 
+int ControlUSB(int reader_index, int requesttype, int request, int value,
+	unsigned char *bytes, unsigned int size);
+
+#endif
