@@ -18,7 +18,7 @@
 */
 
 /*
- * $Id: commands.c 2511 2007-05-07 09:31:13Z rousseau $
+ * $Id: commands.c 2602 2007-08-02 20:04:12Z rousseau $
  */
 
 #include <string.h>
@@ -378,10 +378,7 @@ RESPONSECODE SecurePINVerify(unsigned int reader_index,
 		return_value = CmdEscape(reader_index, cmd_tmp, sizeof(cmd_tmp),
 			res_tmp, &res_length);
 		if (return_value != IFD_SUCCESS)
-		{
-			ccid_error(res_tmp[ERROR_OFFSET], __FILE__, __LINE__, __FUNCTION__);
 			return return_value;
-		}
 	}
 
 	i2dw(a - 10, cmd + 1);  /* CCID message length */
@@ -934,7 +931,8 @@ again_status:
 #ifdef O2MICRO_OZ776_PATCH
 		/* the O2MICRO OZ 776 reader sends card absent or mute errors
 		 * when no card is inserted */
-		if (! ((OZ776 == ccid_descriptor->readerID)
+		if (! (((OZ776 == ccid_descriptor->readerID)
+			|| (OZ776_7772 == ccid_descriptor->readerID))
 			&& (buffer[ERROR_OFFSET] == 0xFE)))
 #endif
 		ccid_error(buffer[ERROR_OFFSET], __FILE__, __LINE__, __FUNCTION__);    /* bError */
