@@ -17,13 +17,14 @@
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
 */
 
-/* $Id: ifdhandler.c 2258 2006-12-02 17:13:06Z rousseau $ */
+/* $Id: ifdhandler.c 2648 2007-10-11 13:41:35Z rousseau $ */
 
 #include <stdio.h>
 #include <string.h>
 #include <stdlib.h>
 #include <arpa/inet.h>
 #include "misc.h"
+#include "config.h"
 #include <pcsclite.h>
 #include <ifdhandler.h>
 #include <reader.h>
@@ -31,7 +32,6 @@
 #include "ccid.h"
 #include "defs.h"
 #include "ccid_ifdhandler.h"
-#include "config.h"
 #include "debug.h"
 #include "utils.h"
 #include "commands.h"
@@ -310,7 +310,7 @@ EXTERNAL RESPONSECODE IFDHGetCapabilities(DWORD Lun, DWORD Tag,
 			{
 				*Length = 1;
 				*Value = 1 + get_ccid_descriptor(reader_index) -> bMaxSlotIndex;
-				DEBUG_INFO2("Reader supports %d slots", *Value);
+				DEBUG_INFO2("Reader supports %d slot(s)", *Value);
 			}
 			break;
 
@@ -953,7 +953,10 @@ EXTERNAL RESPONSECODE IFDHTransmitToICC(DWORD Lun, SCARD_IO_HEADER SendPci,
 	rx_length = *RxLength;
 	return_value = CmdXfrBlock(reader_index, TxLength, TxBuffer, &rx_length,
 		RxBuffer, SendPci.Protocol);
-	*RxLength = rx_length;
+	if (IFD_SUCCESS == return_value)
+		*RxLength = rx_length;
+	else
+		*RxLength = 0;
 
 	return return_value;
 } /* IFDHTransmitToICC */
@@ -981,6 +984,7 @@ EXTERNAL RESPONSECODE IFDHControl(DWORD Lun, DWORD dwControlCode,
 	int reader_index;
 
 	DEBUG_INFO3("lun: %X, ControlCode: 0x%X", Lun, dwControlCode);
+	DEBUG_INFO_XXD("Control TxBuffer: ", TxBuffer, TxLength);
 
 	reader_index = LunToReaderIndex(Lun);
 	if ((-1 == reader_index) || (NULL == pdwBytesReturned))
@@ -1067,6 +1071,10 @@ EXTERNAL RESPONSECODE IFDHControl(DWORD Lun, DWORD dwControlCode,
 		*pdwBytesReturned = iBytesReturned;
 	}
 
+	if (IFD_SUCCESS != return_value)
+		*pdwBytesReturned = 0;
+
+	DEBUG_INFO_XXD("Control RxBuffer: ", RxBuffer, *pdwBytesReturned);
 	return return_value;
 } /* IFDHControl */
 
