@@ -18,7 +18,7 @@
 */
 
 /*
- * $Id: parse.c 2328 2007-01-10 20:53:44Z rousseau $
+ * $Id: parse.c 2679 2007-11-01 17:16:43Z rousseau $
  */
 
 #include <stdio.h>
@@ -51,6 +51,7 @@ int main(void)
 	static struct usb_bus *busses = NULL;
 	struct usb_bus *bus;
 	struct usb_dev_handle *dev_handle;
+	int nb = 0;
 
 	usb_init();
 	usb_find_busses();
@@ -72,6 +73,7 @@ int main(void)
 		for (dev = bus->devices; dev; dev = dev->next)
 		{
 			struct usb_interface *usb_interface = NULL;
+			int interface;
 
 			/* check if the device has bInterfaceClass == 11 */
 			usb_interface = get_ccid_usb_interface(dev);
@@ -107,11 +109,34 @@ int main(void)
 				continue;
 			}
 
+			interface = usb_interface->altsetting->bInterfaceNumber;
+#ifndef __APPLE__
+			if (usb_claim_interface(dev_handle, interface) < 0)
+			{
+				usb_close(dev_handle);
+				fprintf(stderr, "Can't claim interface %s/%s: %s\n",
+						bus->dirname, dev->filename, strerror(errno));
+				if (EBUSY == errno)
+				{
+					printf(" \33[01;31mPlease, stop pcscd and retry\33[0m\n\n");
+					return TRUE;
+				}
+				continue;
+			}
+#endif
+
 			ccid_parse_interface_descriptor(dev_handle, dev);
+
+#ifndef __APPLE__
+			usb_release_interface(dev_handle, interface);
+#endif
 			usb_close(dev_handle);
+			nb++;
 		}
 	}
 
+	if ((0 == nb) && (0 != geteuid()))
+		fprintf(stderr, "Can't find any CCID device.\nMaybe you must run parse as root?\n");
 	return 0;
 } /* main */
 
@@ -126,7 +151,7 @@ static int ccid_parse_interface_descriptor(usb_dev_handle *handle,
 {
 	struct usb_interface_descriptor *usb_interface;
 	unsigned char *extra;
-	char buffer[255];
+	unsigned char buffer[256*sizeof(int)];  /* maximum is 256 records */
 
 	/*
 	 * Vendor/model name
@@ -166,6 +191,23 @@ static int ccid_parse_interface_descriptor(usb_dev_handle *handle,
 	printf(" bAlternateSetting: %d\n", usb_interface->bAlternateSetting);
 
 	printf(" bNumEndpoints: %d\n", usb_interface->bNumEndpoints);
+	switch (usb_interface->bNumEndpoints)
+	{
+		case 0:
+			printf("  Control only\n");
+			break;
+		case 1:
+			printf("  Interrupt-IN\n");
+			break;
+		case 2:
+			printf("  bulk-IN and bulk-OUT\n");
+			break;
+		case 3:
+			printf("  bulk-IN, bulk-OUT and Interrupt-IN\n");
+			break;
+		default:
+			printf("  UNKNOWN value\n");
+	}
 
 	printf(" bInterfaceClass: 0x%02X", usb_interface->bInterfaceClass);
 	if (usb_interface->bInterfaceClass == 0x0b)
@@ -184,8 +226,20 @@ static int ccid_parse_interface_descriptor(usb_dev_handle *handle,
 		printf("  UNSUPPORTED SubClass\n");
 
 	printf(" bInterfaceProtocol: %d\n", usb_interface->bInterfaceProtocol);
-	if (usb_interface->bInterfaceProtocol)
-		printf("  UNSUPPORTED InterfaceProtocol\n");
+	switch (usb_interface->bInterfaceProtocol)
+	{
+		case 0:
+			printf("  bulk transfer, optional interrupt-IN\n");
+			break;
+		case 1:
+			printf("  ICCD Version A, Control transfers, (no interrupt-IN)\n");
+			break;
+		case 2:
+			printf("  ICCD Version B, Control transfers, (optional interrupt-IN)\n");
+			break;
+		default:
+			printf("  UNSUPPORTED InterfaceProtocol\n");
+	}
 
 	printf(" iInterface: %d\n", usb_interface->iInterface);
 
@@ -243,7 +297,6 @@ static int ccid_parse_interface_descriptor(usb_dev_handle *handle,
 	printf("  bNumClockSupported: %d %s\n", extra[18],
 		extra[18] ? "" : "(will use whatever is returned)");
 	{
-		unsigned char buffer[256*sizeof(int)];  /* maximum is 256 records */
 		int n;
 
 		/* See CCID 3.7.2 page 25 */
@@ -252,13 +305,20 @@ static int ccid_parse_interface_descriptor(usb_dev_handle *handle,
 			0x02, /* GET CLOCK FREQUENCIES */
 			0x00, /* value */
 			usb_interface->bInterfaceNumber, /* interface */
-			(char *)buffer,
+			buffer,
 			sizeof(buffer),
 			2 * 1000);
 
 		/* we got an error? */
 		if (n <= 0)
-			printf("   IFD does not support GET CLOCK FREQUENCIES request\n");
+		{
+			printf("   IFD does not support GET CLOCK FREQUENCIES request: %s\n", strerror(errno));
+			if (EBUSY == errno)
+			{
+				printf("   \33[01;31mPlease, stop pcscd and retry\33[0m\n\n");
+				return TRUE;
+			}
+		}
 		else
 			if (n % 4) 	/* not a multiple of 4 */
 				printf("   wrong size for GET CLOCK FREQUENCIES: %d\n", n);
@@ -286,7 +346,6 @@ static int ccid_parse_interface_descriptor(usb_dev_handle *handle,
 	printf("  bNumDataRatesSupported: %d %s\n", extra[27],
 		extra[27] ? "" : "(will use whatever is returned)");
 	{
-		unsigned char buffer[256*sizeof(int)];  /* maximum is 256 records */
 		int n;
 
 		/* See CCID 3.7.3 page 25 */
@@ -295,13 +354,14 @@ static int ccid_parse_interface_descriptor(usb_dev_handle *handle,
 			0x03, /* GET DATA RATES */
 			0x00, /* value */
 			usb_interface->bInterfaceNumber, /* interface */
-			(char *)buffer,
+			buffer,
 			sizeof(buffer),
 			2 * 1000);
 
 		/* we got an error? */
 		if (n <= 0)
-			printf("   IFD does not support GET_DATA_RATES request\n");
+			printf("   IFD does not support GET_DATA_RATES request: %s\n",
+				strerror(errno));
 		else
 			if (n % 4) 	/* not a multiple of 4 */
 				printf("   wrong size for GET_DATA_RATES: %d\n", n);
@@ -368,6 +428,8 @@ static int ccid_parse_interface_descriptor(usb_dev_handle *handle,
 		printf("   ..02.. NAD value other than 00 accepted (T=1)\n");
 	if (extra[41] & 0x04)
 		printf("   ..04.. Automatic IFSD exchange as first exchange (T=1)\n");
+	if (extra[41] & 0x08)
+		printf("   ..08.. Unknown (ICCD?)\n");
 	switch (extra[42] & 0x07)
 	{
 		case 0x00:
