@@ -18,7 +18,7 @@
 */
 
 /*
- * $Id: utils.h 1399 2005-03-14 18:56:53Z rousseau $
+ * $Id: utils.h 2121 2006-08-03 13:12:20Z rousseau $
  */
 
 #ifndef TRUE
@@ -29,5 +29,5 @@
 void InitReaderIndex(void);
 int GetNewReaderIndex(const DWORD Lun);
 int LunToReaderIndex(DWORD Lun);
-int ReleaseReaderIndex(const int index);
+void ReleaseReaderIndex(const int index);
 
